package com.hzya.frame.datasource;

import com.alibaba.druid.pool.DruidDataSource;
import com.baomidou.dynamic.datasource.DynamicRoutingDataSource;
import com.hzya.frame.web.entity.BaseEntity;
import com.hzya.frame.web.exception.BaseSystemException;
import org.apache.ibatis.mapping.DatabaseIdProvider;
import org.apache.ibatis.mapping.VendorDatabaseIdProvider;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Lazy;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * @Auther: xiang2lin
 * @Date: 2020/3/31 10:25
 * 数据源工具类
 */
//@Component
//@Lazy(value = false)
public class DataSourceUtil {
    //DATASOURCECODE
    public static final ThreadLocal<BaseEntity> myContextThreadLocal = new ThreadLocal<>();
    private static final Logger logger = LoggerFactory.getLogger(DataSourceUtil.class);
    //SqlSessionFactory容器
    private static final Map<String, SqlSessionFactory> SQLSESSIONFACTORYMAP = new HashMap<>();
    //dataSource容器
    private final Map<String, DruidDataSource> DATASOURCEMAP = new HashMap<>();
    // 通过依赖注入获取 DynamicRoutingDataSource 对象
    @Resource
    private  DynamicRoutingDataSource ds;
    /****
     * 根据dataSourceKey 打开 SqlSession ,如果获取不到SqlSessionFactory 会抛出异常
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2024-02-22 14:50
     * @param
     * @return org.apache.ibatis.session.SqlSession
     **/

    public synchronized static SqlSession getSqlSession() {
        String dsKey = "master"; // 根据业务逻辑确定要使用的数据源名称
        BaseEntity entity = DataSourceUtil.myContextThreadLocal.get();
        if(null != entity ){
            dsKey = entity.getDataSourceCode();
        }
        // 通过 DynamicRoutingDataSource 获取当前数据源
//       ds.getDataSource(dsKey);

        SqlSessionFactory factory = getSqlSessionFactoryByCache(dsKey);
        if (null == factory) {
            throw new BaseSystemException("根据dataSourceKey获取SqlSessionFactory 失败!请联系管理员");
        } else {
            return factory.openSession();
        }
    }


    /****
     * 根据dataSourceKey获取缓存内的SqlSessionFactory,如果获取不到会抛出异常信息
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2024-02-22 14:44
     * @param
     * @return org.apache.ibatis.session.SqlSessionFactory
     **/
    public static SqlSessionFactory getSqlSessionFactoryByCache(String dataSourceKey) {
        SqlSessionFactory sqlSessionFactory = SQLSESSIONFACTORYMAP.get(dataSourceKey);
        if (null == sqlSessionFactory) {
            throw new BaseSystemException("根据当前dataSourceKey获取SqlSessionFactory失败,请联系系统管理员！dataSourceKey:" + dataSourceKey);
        }
        return sqlSessionFactory;
    }

    /****
     * 根据 dataSource 创建SqlSessionFactory,并放入map缓存 dataSourceKey
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2024-02-22 14:38
     * @param
     * @return org.apache.ibatis.session.SqlSessionFactory
     **/
    public static SqlSessionFactory createSqlSessionFactoryToCache(String dataSourceKey, DataSource dataSource) {
        SqlSessionFactory sqlSessionFactory = createSqlSessionFactory(dataSource);
        SQLSESSIONFACTORYMAP.put(dataSourceKey, sqlSessionFactory);
        return sqlSessionFactory;
    }

    //创建sqlSessionFactory
    public static SqlSessionFactory createSqlSessionFactory(DataSource dataSource) throws BaseSystemException {
        try {
            if (null != dataSource) {
                SqlSessionFactoryBean sqlSessionFactoryBean = new SqlSessionFactoryBean();
                //数据源
                sqlSessionFactoryBean.setDataSource(dataSource);
                //mybatis的config文件
                PathMatchingResourcePatternResolver configLocationResolver = new PathMatchingResourcePatternResolver();
                sqlSessionFactoryBean.setConfigLocation(configLocationResolver.getResource("classpath:mybatis/mybatis-config.xml"));
                //xml
                PathMatchingResourcePatternResolver resourcePatternResolver = new PathMatchingResourcePatternResolver();
                sqlSessionFactoryBean.setMapperLocations(resourcePatternResolver.getResources("classpath*:com/hzya/**/*.xml"));
                sqlSessionFactoryBean.setDatabaseIdProvider(getDatabaseIdProvider());

                return sqlSessionFactoryBean.getObject();
            } else {
                logger.info("======================创建sqlSessionFactory失败===========================");
                throw new BaseSystemException("创建sqlSessionFactory失败");
            }
        } catch (Exception e) {
            logger.info("======================创建sqlSessionFactory失败===========================");
            throw new BaseSystemException("创建sqlSessionFactory失败" + e.getMessage());
        }
    }

    public static DatabaseIdProvider getDatabaseIdProvider() {
        //DBMS 后缀
        //Oracle oracle
        //DB2 db2
        //MSSQLServer mssql
        //MySQL mysql
        //PostgreSQL postgre
        //Firebird firebird
        //MaxDB maxdb
        //HSQL hsql
        //Derby derby
        //H2 h2
        //Sybase(1.0.43以后) sybase
        VendorDatabaseIdProvider dbProvider = new VendorDatabaseIdProvider();
        Properties setPro = new Properties();
        //此处可以添加其他的数据库型号.如MySQL这类厂商标识一定不要写错！！
        setPro.setProperty("MySQL", "mysql");  //MySQL必须这样写，但是mysql可以随意写，后面代码同此处.
        setPro.setProperty("Oracle", "oracle");
        setPro.setProperty("DM DBMS", "dm");
        dbProvider.setProperties(setPro);
        return dbProvider;
    }

    public Map<String, DruidDataSource> getDataSourceMap() {
        return DATASOURCEMAP;
    }

}
